# -*- coding: utf-8 -*-
""" asammdf *Signal* class module for time correct signal processing """

import numpy as np
import warnings

from .utils import MdfException
from .version import __version__


class SignalConversions(object):
    """
    types of generic conversions found in the `Signal` conversion attribute.
    This holds all the conversion types found in the mdf versions 3 and 4
    """

    CONVERSION_NONE = 0
    CONVERSION_LINEAR = 1
    CONVERSION_RATIONAL = 2
    CONVERSION_ALGEBRAIC = 3
    CONVERSION_POLYNOMIAL = 4
    CONVERSION_TAB = 5
    CONVERSION_TABI = 6
    CONVERSION_TABX = 7
    CONVERSION_RTAB = 8
    CONVERSION_RTABX = 9
    CONVERSION_TTAB = 10
    CONVERSION_TRANS = 11
    CONVERSION_EXPO = 12
    CONVERSION_LOGH = 13


class Signal(object):
    """
    The *Signal* represents a hannel described by it's samples and timestamps.
    It can perform aritmethic operations agains other *Signal* or numeric types.
    The operations are computed in respect to the timestamps (time correct).
    The non-float signals are not interpolated, instead the last value relative
    to the current timestamp is used.
    *samples*, *timstamps* and *name* are mandatory arguments.

    Parameters
    ----------
    samples : numpy.array | list | tuple
        signal samples
    timestamps : numpy.array | list | tuple
        signal timestamps
    unit : str
        signal unit
    name : str
        signal name
    conversion : dict
        dict that contains extra conversionrmation about the signal ,
        default *None*
    comment : str
        signal comment, default ''
    raw : bool
        signal samples are raw values, with no physical conversion applied

    """

    __slots__ = [
        'samples',
        'timestamps',
        'unit',
        'name',
        'conversion',
        'comment',
        '_plot_axis',
        'raw',
    ]

    def __init__(self,
                 samples=None,
                 timestamps=None,
                 unit='',
                 name='',
                 conversion=None,
                 comment='',
                 raw=False):

        if samples is None or timestamps is None or name == '':
            message = ('"samples", "timestamps" and "name" are mandatory '
                       'for Signal class __init__: samples={}\n'
                       'timestamps={}\nname={}')
            raise MdfException(message.format(samples, timestamps, name))
        else:
            if isinstance(samples, (list, tuple)):
                samples = np.array(samples)
            if isinstance(timestamps, (list, tuple)):
                timestamps = np.array(timestamps, dtype=np.float64)
            if not samples.shape[0] == timestamps.shape[0]:
                message = 'samples and timestamps length missmatch ({} vs {})'
                message = message.format(samples.shape[0], timestamps.shape[0])
                raise MdfException(message)
            self.samples = samples
            self.timestamps = timestamps
            self.unit = unit
            self.name = name
            self.conversion = conversion
            self.comment = comment
            self._plot_axis = None
            self.raw = raw

#    def physical(self):
#        """ get Signal with physical conversion appplied
#        to its samples
#
#        """
#        if self.raw:
#            pass
#        else:
#            return self

    def __str__(self):
        string = """<Signal {}:
\tsamples={}
\ttimestamps={}
\tunit="{}"
\tconversion={}
\tcomment="{}"
\traw={}>
"""
        return string.format(
            self.name,
            self.samples,
            self.timestamps,
            self.unit,
            self.conversion,
            self.comment,
            self.raw,
        )

    def __repr__(self):
        string = (
            'Signal(name={}, samples={}, timestamps={}, '
            'unit={}, conversion={}, comment={}, raw={})'
        )
        return string.format(
            self.name,
            repr(self.samples),
            repr(self.timestamps),
            self.unit,
            self.conversion,
            self.comment,
            self.raw,
        )

    def plot(self):
        """ plot Signal samples """
        try:
            import matplotlib.pyplot as plt
            from mpl_toolkits.mplot3d import axes3d
            from matplotlib.widgets import Slider
        except ImportError:
            warnings.warn("Signal plotting requires matplotlib")
            return

        if len(self.samples.shape) <= 1 and self.samples.dtype.names is None:
            fig = plt.figure()
            fig.canvas.set_window_title(self.name)
            fig.text(0.95, 0.05, 'asammdf {}'.format(__version__),
                     fontsize=8, color='red',
                     ha='right', va='top', alpha=0.5)

            if self.comment:
                comment = self.comment.replace('$', '')
                plt.title('{}\n({})'.format(self.name, comment))
            else:
                plt.title(self.name)
            plt.xlabel('Time [s]')
            plt.ylabel('[{}]'.format(self.unit))
            plt.plot(self.timestamps, self.samples, 'b')
            plt.plot(self.timestamps, self.samples, 'b.')
            plt.grid(True)
            plt.show()
        else:
            try:
                names = self.samples.dtype.names
                if self.samples.dtype.names is None or len(names) == 1:

                    if names:
                        samples = self.samples[names[0]]
                    else:
                        samples = self.samples

                    shape = samples.shape[1:]

                    fig = plt.figure()
                    fig.canvas.set_window_title(self.name)
                    fig.text(
                        0.95,
                        0.05,
                        'asammdf {}'.format(__version__),
                        fontsize=8,
                        color='red',
                        ha='right',
                        va='top',
                        alpha=0.5,
                    )

                    if self.comment:
                        comment = self.comment.replace('$', '')
                        plt.title('{}\n({})'.format(self.name, comment))
                    else:
                        plt.title(self.name)

                    ax = fig.add_subplot(111, projection='3d')

                    # Grab some test data.
                    X = np.array(range(shape[1]))
                    Y = np.array(range(shape[0]))
                    X, Y = np.meshgrid(X, Y)

                    Z = samples[0]

                    # Plot a basic wireframe.
                    self._plot_axis = ax.plot_wireframe(X, Y, Z,
                                                        rstride=1, cstride=1)

                    # Place Sliders on Graph
                    ax_a = plt.axes([0.25, 0.1, 0.65, 0.03])

                    # Create Sliders & Determine Range
                    sa = Slider(ax_a,
                                'Time [s]',
                                self.timestamps[0],
                                self.timestamps[-1],
                                valinit=self.timestamps[0])

                    def update(val):
                        self._plot_axis.remove()
                        idx = np.searchsorted(self.timestamps,
                                              sa.val,
                                              side='right')
                        Z = samples[idx-1]
                        self._plot_axis = ax.plot_wireframe(
                            X,
                            Y,
                            Z,
                            rstride=1,
                            cstride=1,
                        )
                        fig.canvas.draw_idle()

                    sa.on_changed(update)

                    plt.show()

                else:
                    fig = plt.figure()
                    fig.canvas.set_window_title(self.name)
                    fig.text(
                        0.95,
                        0.05,
                        'asammdf {}'.format(__version__),
                        fontsize=8,
                        color='red',
                        ha='right',
                        va='top',
                        alpha=0.5,
                    )

                    if self.comment:
                        comment = self.comment.replace('$', '')
                        plt.title('{}\n({})'.format(self.name, comment))
                    else:
                        plt.title(self.name)

                    ax = fig.add_subplot(111, projection='3d')

                    samples = self.samples[names[0]]
                    axis1 = self.samples[names[1]]
                    axis2 = self.samples[names[2]]

                    # Grab some test data.
                    X, Y = np.meshgrid(axis2[0], axis1[0])

                    Z = samples[0]

                    # Plot a basic wireframe.
                    self._plot_axis = ax.plot_wireframe(
                        X,
                        Y,
                        Z,
                        rstride=1,
                        cstride=1,
                    )

                    # Place Sliders on Graph
                    ax_a = plt.axes([0.25, 0.1, 0.65, 0.03])

                    # Create Sliders & Determine Range
                    sa = Slider(ax_a,
                                'Time [s]',
                                self.timestamps[0],
                                self.timestamps[-1],
                                valinit=self.timestamps[0])

                    def update(val):
                        self._plot_axis.remove()
                        idx = np.searchsorted(self.timestamps,
                                              sa.val,
                                              side='right')
                        Z = samples[idx-1]
                        X, Y = np.meshgrid(axis2[idx-1], axis1[idx-1])
                        self._plot_axis = ax.plot_wireframe(
                            X,
                            Y,
                            Z,
                            rstride=1,
                            cstride=1,
                        )
                        fig.canvas.draw_idle()

                    sa.on_changed(update)

                    plt.show()

            except Exception as err:
                print(err)

    def cut(self, start=None, stop=None):
        """
        Cuts the signal according to the *start* and *stop* values, by using
        the insertion indexes in the signal's *time* axis.

        Parameters
        ----------
        start : float
            start timestamp for cutting
        stop : float
            stop timestamp for cutting

        Returns
        -------
        result : Signal
            new *Signal* cut from the original

        Examples
        --------
        >>> new_sig = old_sig.cut(1.0, 10.5)
        >>> new_sig.timestamps[0], new_sig.timestamps[-1]
        0.98, 10.48

        """

        if start is None and stop is None:
            # return the channel uncut
            result = self

        else:
            if start is None:
                # cut from beggining to stop
                stop = np.searchsorted(self.timestamps, stop, side='right')
                if stop:
                    result = Signal(
                        self.samples[: stop],
                        self.timestamps[:stop],
                        self.unit,
                        self.name,
                        self.conversion,
                        self.comment,
                        self.raw,
                    )
                else:
                    result = Signal(
                        np.array([]),
                        np.array([]),
                        self.unit,
                        self.name,
                        self.conversion,
                        self.comment,
                        self.raw,
                    )

            elif stop is None:
                # cut from start to end
                start = np.searchsorted(self.timestamps, start, side='left')
                result = Signal(
                    self.samples[start:],
                    self.timestamps[start:],
                    self.unit,
                    self.name,
                    self.conversion,
                    self.comment,
                    self.raw,
                )

            else:
                # cut between start and stop
                start_ = np.searchsorted(self.timestamps, start, side='left')
                stop_ = np.searchsorted(self.timestamps, stop, side='right')
                if stop_ == start_:

                    if (len(self.timestamps)
                            and stop >= self.timestamps[0]
                            and start <= self.timestamps[-1]):
                        # start and stop are found between 2 signal samples
                        # so return the previous sample
                        result = Signal(
                            self.samples[start_: start_ + 1],
                            self.timestamps[start_: start_ + 1],
                            self.unit,
                            self.name,
                            self.conversion,
                            self.comment,
                            self.raw,
                        )
                    else:
                        # signal is empty or start and stop are outside the
                        # signal time base
                        result = Signal(
                            np.array([]),
                            np.array([]),
                            self.unit,
                            self.name,
                            self.conversion,
                            self.comment,
                            self.raw,
                        )
                else:
                    result = Signal(
                        self.samples[start_: stop_],
                        self.timestamps[start_: stop_],
                        self.unit,
                        self.name,
                        self.conversion,
                        self.comment,
                        self.raw,
                    )
        return result

    def extend(self, other):
        """ extend signal with samples from another signal

        Parameters
        ----------
        other : Signal

        Returns
        -------
        signal : Signal
            new extended *Signal*

        """
        if len(self.timestamps):
            last_stamp = self.timestamps[-1]
            delta = last_stamp / len(self) + last_stamp
        else:
            last_stamp = 0
            delta = 0
        if len(other):
            other_first_sample = other.timestamps[0]
            if last_stamp >= other_first_sample:
                timestamps = other.timestamps + delta - other_first_sample
            else:
                timestamps = other.timestamps

            result = Signal(
                np.append(self.samples, other.samples),
                np.append(self.timestamps, timestamps),
                self.unit,
                self.name,
                self.conversion,
                self.comment,
                self.raw,
            )
        else:
            result = self

        return result

    def interp(self, new_timestamps):
        """ returns a new *Signal* interpolated using the *new_timestamps*

        Parameters
        ----------
        new_timestamps : np.array
            timestamps used for interpolation

        Returns
        -------
        signal : Signal
            new interpolated *Signal*

        """
        if self.samples.dtype.kind == 'f':
            s = np.interp(new_timestamps, self.timestamps, self.samples)
        else:
            idx = np.searchsorted(
                self.timestamps,
                new_timestamps,
                side='right',
            )
            idx -= 1
            idx = np.clip(idx, 0, idx[-1])
            s = self.samples[idx]
        return Signal(
            s,
            new_timestamps,
            self.unit,
            self.name,
            self.conversion,
            self.raw,
        )

    def __apply_func(self, other, func_name):
        """ delegate operations to the *samples* attribute, but in a time
        correct manner by considering the *timestamps*

        """

        if isinstance(other, Signal):
            time = np.union1d(self.timestamps, other.timestamps)
            s = self.interp(time).samples
            o = other.interp(time).samples
            func = getattr(s, func_name)
            s = func(o)
        elif other is None:
            s = self.samples
            time = self.timestamps
        else:
            func = getattr(self.samples, func_name)
            s = func(other)
            time = self.timestamps
        return Signal(
            s,
            time,
            self.unit,
            self.name,
            self.conversion,
            self.raw,
        )

    def __pos__(self):
        return self

    def __neg__(self):
        return Signal(
            np.negative(self.samples),
            self.timestamps,
            self.unit,
            self.name,
            self.conversion,
            self.raw,
        )

    def __round__(self, n):
        return Signal(
            np.around(self.samples, n),
            self.timestamps,
            self.unit,
            self.name,
            self.conversion,
            self.raw,
        )

    def __sub__(self, other):
        return self.__apply_func(other, '__sub__')

    def __isub__(self, other):
        return self.__sub__(other)

    def __rsub__(self, other):
        return -self.__sub__(other)

    def __add__(self, other):
        return self.__apply_func(other, '__add__')

    def __iadd__(self, other):
        return self.__add__(other)

    def __radd__(self, other):
        return self.__add__(other)

    def __mul__(self, other):
        return self.__apply_func(other, '__mul__')

    def __imul__(self, other):
        return self.__mul__(other)

    def __rmul__(self, other):
        return self.__mul__(other)

    def __truediv__(self, other):
        return self.__apply_func(other, '__truediv__')

    def __itruediv__(self, other):
        return self.__truediv__(other)

    def __rtruediv__(self, other):
        return self.__apply_func(other, '__rtruediv__')

    def __mod__(self, other):
        return self.__apply_func(other, '__mod__')

    def __pow__(self, other):
        return self.__apply_func(other, '__pow__')

    def __and__(self, other):
        return self.__apply_func(other, '__and__')

    def __or__(self, other):
        return self.__apply_func(other, '__or__')

    def __xor__(self, other):
        return self.__apply_func(other, '__xor__')

    def __invert__(self):
        s = ~self.samples
        time = self.timestamps
        return Signal(
            s,
            time,
            self.unit,
            self.name,
            self.conversion,
            self.raw,
        )

    def __lshift__(self, other):
        return self.__apply_func(other, '__lshift__')

    def __rshift__(self, other):
        return self.__apply_func(other, '__rshift__')

    def __lt__(self, other):
        return self.__apply_func(other, '__lt__')

    def __le__(self, other):
        return self.__apply_func(other, '__le__')

    def __gt__(self, other):
        return self.__apply_func(other, '__gt__')

    def __ge__(self, other):
        return self.__apply_func(other, '__ge__')

    def __eq__(self, other):
        return self.__apply_func(other, '__eq__')

    def __ne__(self, other):
        return self.__apply_func(other, '__ne__')

    def __iter__(self):
        for item in (
                self.samples,
                self.timestamps,
                self.unit,
                self.name):
            yield item

    def __reversed__(self):
        return enumerate(zip(reversed(self.samples),
                             reversed(self.timestamps)))

    def __len__(self):
        return len(self.samples)

    def __abs__(self):
        return Signal(
            np.fabs(self.samples),
            self.timestamps,
            self.unit,
            self.name,
            self.conversion,
            self.raw,
        )

    def __getitem__(self, val):
        return self.samples[val]

    def __setitem__(self, idx, val):
        self.samples[idx] = val

    def astype(self, np_type):
        """ returns new *Signal* with samples of dtype *np_type*

        Parameters
        ----------
        np_type : np.dtype
            new numpy dtye

        Returns
        -------
        signal : Signal
            new *Signal* with the samples of *np_type* dtype

        """
        return Signal(
            self.samples.astype(np_type),
            self.timestamps,
            self.unit,
            self.name,
            self.conversion,
            self.raw,
        )


if __name__ == '__main__':
    pass



#!/usr/bin/env python
from __future__ import print_function
import unittest

import numpy as np

from utils import MEMORY
from asammdf import MDF, MDF4, Signal

CHANNEL_LEN = 100000


class TestMDF4(unittest.TestCase):

    def test_measurement(self):
        self.assertTrue(MDF4)

    def test_read_mdf4_00(self):

        seed = np.random.randint(0, 2**31)

        np.random.seed(seed)
        print('Read 4.00 using seed =', seed)

        sig_int = Signal(
            np.random.randint(-2**31, 2**31, CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Integer Channel',
            unit='unit1',
        )

        sig_float = Signal(
            np.random.random(CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Float Channel',
            unit='unit2',
        )

        for memory in MEMORY:

            with MDF(version='4.00', memory=memory) as mdf:
                mdf.append([sig_int, sig_float], common_timebase=True)
                outfile = mdf.save('tmp', overwrite=True)

            with MDF(outfile, memory=memory) as mdf:
                ret_sig_int = mdf.get(sig_int.name)
                ret_sig_float = mdf.get(sig_float.name)

            self.assertTrue(np.array_equal(ret_sig_int.samples,
                                           sig_int.samples))
            self.assertTrue(np.array_equal(ret_sig_float.samples,
                                           sig_float.samples))

    def test_read_mdf4_10(self):

        seed = np.random.randint(0, 2**31)

        np.random.seed(seed)
        print('Read 4.10 using seed =', seed)

        sig_int = Signal(
            np.random.randint(-2**31, 2**31, CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Integer Channel',
            unit='unit1',
        )

        sig_float = Signal(
            np.random.random(CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Float Channel',
            unit='unit2',
        )

        for memory in MEMORY:
            with MDF(version='4.10', memory=memory) as mdf:
                mdf.append([sig_int, sig_float], common_timebase=True)
                outfile = mdf.save('tmp', overwrite=True)

            with MDF(outfile, memory=memory) as mdf:
                ret_sig_int = mdf.get(sig_int.name)
                ret_sig_float = mdf.get(sig_float.name)

            self.assertTrue(np.array_equal(ret_sig_int.samples,
                                           sig_int.samples))
            self.assertTrue(np.array_equal(ret_sig_float.samples,
                                           sig_float.samples))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""
Main test function to execute all tests found in the current directory
"""
import sys
import xmlrunner

try:
    import unittest2 as unittest
except ImportError:
    import unittest


def main():
    tests = unittest.TestLoader().discover('.', 'test_*.py')
    testResult = xmlrunner.XMLTestRunner(output='test-reports').run(tests)

    return not testResult.wasSuccessful()

if __name__ == '__main__':
    sys.exit(main())


#!/usr/bin/env python
from __future__ import print_function
import unittest

import numpy as np

from utils import MEMORY
from asammdf import MDF, MDF2, MDF3, Signal, configure

CHANNEL_LEN = 10000

configure(
        split_data_blocks=True,
        split_threshold=260,
        overwrite=True)


class TestMDF23(unittest.TestCase):

    def test_measurement(self):
        self.assertTrue(MDF2)
        self.assertTrue(MDF3)

    def test_read_mdf2_00(self):

        seed = np.random.randint(0, 2**31)

        np.random.seed(seed)
        print('Read 2.00 using seed =', seed)

        sig_int = Signal(
            np.random.randint(-2**15, -1, CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Integer Channel',
            unit='unit1',
        )

        sig_float = Signal(
            np.random.random(CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Float Channel',
            unit='unit2',
        )

        for memory in MEMORY:
            print(memory)

            with MDF(version='2.00', memory=memory) as mdf:
                mdf.append([sig_int, sig_float], common_timebase=True)
                outfile = mdf.save('tmp', overwrite=True)

            with MDF(outfile, memory=memory) as mdf:
                ret_sig_int = mdf.get(sig_int.name)
                ret_sig_float = mdf.get(sig_float.name)

            self.assertTrue(np.array_equal(ret_sig_int.samples,
                                           sig_int.samples))
            self.assertTrue(np.array_equal(ret_sig_float.samples,
                                           sig_float.samples))

    def test_read_mdf2_14(self):

        seed = np.random.randint(0, 2**31)

        np.random.seed(seed)
        print('Read 2.14 using seed =', seed)

        sig_int = Signal(
            np.random.randint(-2**29, 2**29, CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Integer Channel',
            unit='unit1',
        )

        sig_float = Signal(
            np.random.random(CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Float Channel',
            unit='unit2',
        )

        for memory in MEMORY:
            print(memory)
            with MDF(version='2.14', memory=memory) as mdf:
                mdf.append([sig_int, sig_float], common_timebase=True)
                outfile = mdf.save('tmp', overwrite=True)

            with MDF(outfile, memory=memory) as mdf:
                ret_sig_int = mdf.get(sig_int.name)
                ret_sig_float = mdf.get(sig_float.name)

            self.assertTrue(np.array_equal(ret_sig_int.samples,
                                           sig_int.samples))
            self.assertTrue(np.array_equal(ret_sig_float.samples,
                                           sig_float.samples))

    def test_read_mdf3_00(self):

        seed = np.random.randint(0, 2**31)

        np.random.seed(seed)
        print('Read 3.00 using seed =', seed)

        sig_int = Signal(
            np.random.randint(-2**16, 2**16, CHANNEL_LEN, np.int32),
            np.arange(CHANNEL_LEN),
            name='Integer Channel',
            unit='unit1',
        )

        sig_float = Signal(
            np.random.random(CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Float Channel',
            unit='unit2',
        )

        for memory in MEMORY:
            print(memory)

            with MDF(version='3.00', memory=memory) as mdf:
                mdf.append([sig_int, sig_float], common_timebase=True)
                outfile = mdf.save('tmp', overwrite=True)

            with MDF(outfile, memory=memory) as mdf:
                ret_sig_int = mdf.get(sig_int.name)
                ret_sig_float = mdf.get(sig_float.name)

            self.assertTrue(np.array_equal(ret_sig_int.samples,
                                           sig_int.samples))
            self.assertTrue(np.array_equal(ret_sig_float.samples,
                                           sig_float.samples))

    def test_read_mdf3_10(self):

        seed = np.random.randint(0, 2**31)

        np.random.seed(seed)
        print('Read 3.10 using seed =', seed)

        sig_int = Signal(
            np.random.randint(-2**9, 2**7, CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Integer Channel',
            unit='unit1',
        )

        sig_float = Signal(
            np.random.random(CHANNEL_LEN),
            np.arange(CHANNEL_LEN),
            name='Float Channel',
            unit='unit2',
        )

        for memory in MEMORY:
            with MDF(version='3.10', memory=memory) as mdf:
                mdf.append([sig_int, sig_float], common_timebase=True)
                outfile = mdf.save('tmp', overwrite=True)

            print(memory)

            with MDF(outfile, memory=memory) as mdf:
                ret_sig_int = mdf.get(sig_int.name)
                ret_sig_float = mdf.get(sig_float.name)

            self.assertTrue(np.array_equal(ret_sig_int.samples,
                                           sig_int.samples))
            self.assertTrue(np.array_equal(ret_sig_float.samples,
                                           sig_float.samples))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
from __future__ import print_function
import os
import random
import sys
import unittest
import shutil
import urllib
from itertools import product
from zipfile import ZipFile


import numpy as np

from utils import (
    CHANNELS_DEMO,
    CHANNELS_ARRAY,
    COMMENTS,
    MEMORY,
    UNITS,
)
from asammdf import MDF, SUPPORTED_VERSIONS

SUPPORTED_VERSIONS = SUPPORTED_VERSIONS[1:]

CHANNEL_LEN = 100000


class TestMDF(unittest.TestCase):

    def test_measurement(self):
        self.assertTrue(MDF)

    @classmethod
    def setUpClass(cls):
        PYVERSION = sys.version_info[0]

        url = 'https://github.com/danielhrisca/asammdf/files/1594267/test.demo.zip'
        if PYVERSION == 3:
            urllib.request.urlretrieve(url, 'test.zip')
        else:
            urllib.urlretrieve(url, 'test.zip')
        ZipFile(r'test.zip').extractall('tmpdir_demo')

        url = 'https://github.com/danielhrisca/asammdf/files/1592123/test.arrays.zip'
        if PYVERSION == 3:
            urllib.request.urlretrieve(url, 'test.zip')
        else:
            urllib.urlretrieve(url, 'test.zip')
        ZipFile(r'test.zip').extractall('tmpdir_array')


    @classmethod
    def tearDownClass(cls):
        shutil.rmtree('tmpdir_demo', True)
        shutil.rmtree('tmpdir_array', True)
        os.remove('test.zip')
        for filename in os.listdir(os.getcwd()):
            if os.path.isfile(filename) and filename.startswith('tmp'):
                os.remove(filename)

    def test_read(self):

        print("MDF read tests")

        ret = True

        for enable in (True, False):
            for mdf in os.listdir('tmpdir_demo'):
                for memory in MEMORY:
                    with MDF(os.path.join('tmpdir_demo', mdf), memory=memory) as input_file:
                        if input_file.version == '2.00':
                            continue
                        for name in set(input_file.channels_db) - {'time', 't'}:
                            signal = input_file.get(name)
                            original_samples = CHANNELS_DEMO[name]
                            if signal.samples.dtype.kind == 'f':
                                signal = signal.astype(np.float32)
                            res = np.array_equal(signal.samples, original_samples)
                            if not res:
                                ret = False

            self.assertTrue(ret)

    def test_get_channel_comment_v4(self):
        print("MDF get channel comment tests")

        ret = True

        for mdf in os.listdir('tmpdir_demo'):
            for memory in MEMORY:
                with MDF(os.path.join('tmpdir_demo', mdf), memory=memory) as input_file:
                    if input_file.version < '4.00':
                        continue
                    print(mdf, memory)
                    for channel_name, original_comment in COMMENTS.items():
                        comment = input_file.get_channel_comment(channel_name)
                        if comment != original_comment:
                            ret = False

        self.assertTrue(ret)

    def test_get_channel_units(self):
        print("MDF get channel units tests")

        ret = True

        for mdf in os.listdir('tmpdir_demo'):
            for memory in MEMORY:
                with MDF(os.path.join('tmpdir_demo', mdf), memory=memory) as input_file:
                    if input_file.version == '2.00':
                        continue
                    print(mdf, memory)
                    for channel_name, original_unit in UNITS.items():
                        comment = input_file.get_channel_unit(channel_name)
                        if comment != original_unit:
                            ret = False

        self.assertTrue(ret)


    def test_read_array(self):

        print("MDF read array tests")

        ret = True

        for mdf in os.listdir('tmpdir_array'):
            for memory in MEMORY:
                with MDF(os.path.join('tmpdir_array', mdf), memory=memory) as input_file:
                    if input_file.version == '2.00':
                        continue
                    for name in set(input_file.channels_db) - {'time', 't'}:
                        signal = input_file.get(name)
                        original_samples = CHANNELS_ARRAY[name]
                        res = np.array_equal(signal.samples, original_samples)
                        if not res:
                            ret = False

        self.assertTrue(ret)

    def test_convert(self):
        print("MDF convert tests")

        for out in SUPPORTED_VERSIONS:
            for mdfname in os.listdir('tmpdir_demo'):
                for memory in MEMORY:
                    input_file = os.path.join('tmpdir_demo', mdfname)
                    if MDF(input_file).version == '2.00':
                        continue
                    print(input_file, memory, out)
                    with MDF(input_file, memory=memory) as mdf:
                        outfile = mdf.convert(out, memory=memory).save('tmp',
                                                             overwrite=True)

                    equal = True

                    with MDF(input_file, memory=memory) as mdf, \
                            MDF(outfile, memory=memory) as mdf2:

                        for name in set(mdf.channels_db) - {'t', 'time'}:
                            original = mdf.get(name)
                            converted = mdf2.get(name)
                            if not np.array_equal(
                                    original.samples,
                                    converted.samples):
                                print('@'*80)
                                print(input_file, out, memory, name)
                                print(original)
                                print(converted)
                                equal = False
                            if not np.array_equal(
                                    original.timestamps,
                                    converted.timestamps):
                                equal = False

                    self.assertTrue(equal)

    def test_merge(self):
        print("MDF merge tests")

        for out in SUPPORTED_VERSIONS:
            for mdfname in os.listdir('tmpdir_demo'):
                for memory in MEMORY:

                    input_file = os.path.join('tmpdir_demo', mdfname)
                    if '2.00' in input_file:
                        continue
                    files = [input_file, ] * 4

                    outfile = MDF.merge(files, out, memory).save('tmp')

                    equal = True

                    with MDF(input_file, memory=memory) as mdf, \
                            MDF(outfile, memory=memory) as mdf2:

                        for i, group in enumerate(mdf.groups):
                            for j, _ in enumerate(group['channels'][1:], 1):
                                original = mdf.get(group=i, index=j)
                                converted = mdf2.get(group=i, index=j)
                                if not np.array_equal(
                                        np.tile(original.samples, 4),
                                        converted.samples):
                                    equal = False
                                    print('='*80)
                                    print(input_file, outfile, memory)
                                    print(original)
                                    print(converted)
                                    print('^'*80)
                                    raise Exception()

                    self.assertTrue(equal)

    def test_merge_array(self):
        print("MDF merge array tests")

        for out in (version for version in SUPPORTED_VERSIONS if version >= '4.00'):
            for mdfname in os.listdir('tmpdir_array'):
                for memory in MEMORY:
                    input_file = os.path.join('tmpdir_array', mdfname)
                    files = [input_file, ] * 4

                    outfile = MDF.merge(files, out, memory).save('tmp', overwrite=True)

                    equal = True

                    with MDF(input_file, memory=memory) as mdf, \
                            MDF(outfile, memory=memory) as mdf2:

                        for i, group in enumerate(mdf.groups):
                            for j, _ in enumerate(group['channels'][1:], 1):
                                original = mdf.get(group=i, index=j)
                                converted = mdf2.get(group=i, index=j)
                                if not np.array_equal(
                                        np.tile(original.samples, 4),
                                        converted.samples):
                                    equal = False

                    self.assertTrue(equal)

    def test_cut_absolute(self):
        print("MDF cut absolute tests")

        for mdfname in os.listdir('tmpdir_demo'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_demo', mdfname)

                if '2.00' in input_file:
                    continue
                print(input_file, memory)

                outfile1 = MDF(input_file, memory=memory).cut(stop=2).save('tmp1', overwrite=True)
                outfile2 = MDF(input_file, memory=memory).cut(start=2, stop=6).save('tmp2', overwrite=True)
                outfile3 = MDF(input_file, memory=memory).cut(start=6).save('tmp3', overwrite=True)

                outfile = MDF.merge(
                    [outfile1, outfile2, outfile3],
                    MDF(input_file, memory='minimum').version,
                ).save('tmp', overwrite=True)

                print('OUT', outfile)

                equal = True

                with MDF(input_file, memory=memory) as mdf, \
                        MDF(outfile, memory=memory) as mdf2:

                    for i, group in enumerate(mdf.groups):
                        for j, _ in enumerate(group['channels'][1:], 1):
                            original = mdf.get(group=i, index=j)
                            converted = mdf2.get(group=i, index=j)
                            if not np.array_equal(
                                    original.samples,
                                    converted.samples):
                                equal = False
                                print('*'*80)
                                print(input_file, memory)
                                print(original)
                                print(mdf.get(group=i, index=j, raw=True))
                                print(converted)
                                print(mdf2.get(group=i, index=j, raw=True))
                                print('='*80)
                                raise Exception()
                            if not np.array_equal(
                                    original.timestamps,
                                    converted.timestamps):
                                equal = False

                self.assertTrue(equal)

    def test_cut_absolute_array(self):
        print("MDF cut absolute array tests")

        for mdfname in os.listdir('tmpdir_array'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_array', mdfname)

                outfile1 = MDF(input_file, memory=memory).cut(stop=2.1).save('tmp1', overwrite=True)
                outfile2 = MDF(input_file, memory=memory).cut(start=2.1, stop=6.1).save('tmp2', overwrite=True)
                outfile3 = MDF(input_file, memory=memory).cut(start=6.1).save('tmp3', overwrite=True)

                outfile = MDF.merge(
                    [outfile1, outfile2, outfile3],
                    MDF(input_file, memory='minimum').version,
                ).save('tmp', overwrite=True)

                equal = True

                with MDF(input_file, memory=memory) as mdf, \
                        MDF(outfile, memory=memory) as mdf2:

                    for i, group in enumerate(mdf.groups):
                        for j, _ in enumerate(group['channels'][1:], 1):
                            original = mdf.get(group=i, index=j)
                            converted = mdf2.get(group=i, index=j)
                            if not np.array_equal(
                                    original.samples,
                                    converted.samples):
                                equal = False
                            if not np.array_equal(
                                    original.timestamps,
                                    converted.timestamps):
                                equal = False

                self.assertTrue(equal)

    def test_cut_relative(self):
        print("MDF cut relative tests")

        for mdfname in os.listdir('tmpdir_demo'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_demo', mdfname)
                if '2.00' in input_file:
                    continue

                outfile1 = MDF(input_file, memory=memory).cut(stop=3, whence=1).save('tmp1', overwrite=True)
                outfile2 = MDF(input_file, memory=memory).cut(start=3, stop=5, whence=1).save('tmp2', overwrite=True)
                outfile3 = MDF(input_file, memory=memory).cut(start=5, whence=1).save('tmp3', overwrite=True)

                outfile = MDF.merge(
                    [outfile1, outfile2, outfile3],
                    MDF(input_file, memory='minimum').version,
                ).save('tmp', overwrite=True)

                equal = True

                with MDF(input_file, memory=memory) as mdf, \
                        MDF(outfile, memory=memory) as mdf2:

                    for i, group in enumerate(mdf.groups):
                        for j, _ in enumerate(group['channels'][1:], 1):
                            original = mdf.get(group=i, index=j)
                            converted = mdf2.get(group=i, index=j)
                            if not np.array_equal(
                                    original.samples,
                                    converted.samples):
                                equal = False
                            if not np.array_equal(
                                    original.timestamps,
                                    converted.timestamps):
                                equal = False

                self.assertTrue(equal)

    def test_cut_relative_array(self):
        print("MDF cut relative array tests")

        for mdfname in os.listdir('tmpdir_array'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_array', mdfname)

                outfile1 = MDF(input_file, memory=memory).cut(stop=3.1, whence=1).save('tmp1', overwrite=True)
                outfile2 = MDF(input_file, memory=memory).cut(start=3.1, stop=5.1, whence=1).save('tmp2', overwrite=True)
                outfile3 = MDF(input_file, memory=memory).cut(start=5.1, whence=1).save('tmp3', overwrite=True)

                outfile = MDF.merge(
                    [outfile1, outfile2, outfile3],
                    MDF(input_file, memory='minimum').version,
                ).save('tmp', overwrite=True)

                equal = True

                with MDF(input_file, memory=memory) as mdf, \
                        MDF(outfile, memory=memory) as mdf2:

                    for i, group in enumerate(mdf.groups):
                        for j, _ in enumerate(group['channels'][1:], 1):
                            original = mdf.get(group=i, index=j)
                            converted = mdf2.get(group=i, index=j)
                            if not np.array_equal(
                                    original.samples,
                                    converted.samples):
                                equal = False
                            if not np.array_equal(
                                    original.timestamps,
                                    converted.timestamps):
                                equal = False

                self.assertTrue(equal)

    def test_filter(self):
        print("MDF filter tests")

        for mdfname in os.listdir('tmpdir_demo'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_demo', mdfname)

                if MDF(input_file, memory=memory).version <= '2.00':
                # if MDF(input_file, memory=memory).version < '4.00':
                    continue

                channels_nr = np.random.randint(1, len(CHANNELS_DEMO) + 1)

                channel_list = random.sample(list(CHANNELS_DEMO), channels_nr)

                filtered_mdf = MDF(input_file, memory=memory).filter(channel_list, memory=memory)

                self.assertTrue((set(filtered_mdf.channels_db) - {'t', 'time'}) == set(channel_list))

                equal = True

                with MDF(input_file, memory=memory) as mdf:
                    print(input_file, memory)

                    for name in channel_list:
                        original = mdf.get(name)
                        filtered = filtered_mdf.get(name)
                        if not np.array_equal(
                                original.samples,
                                filtered.samples):
                            equal = False
                            print('^'*80)
                            print(mdfname, memory)
                            print(original)
                            print(filtered)
                        if not np.array_equal(
                                original.timestamps,
                                filtered.timestamps):
                            equal = False
#                                print('ts', mdfname, memory,  original, filtered)

                self.assertTrue(equal)

    def test_filter_array(self):
        print("MDF filter array tests")

        for mdfname in os.listdir('tmpdir_array'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_array', mdfname)

                channels_nr = np.random.randint(1, len(CHANNELS_ARRAY) + 1)

                channel_list = random.sample(list(CHANNELS_ARRAY), channels_nr)

                filtered_mdf = MDF(input_file, memory=memory).filter(channel_list, memory=memory)

                filtered_mdf.save('fiteed.mf4', overwrite=True)

                target = set(channel_list)
                if 'Int16Array' in target:
                    target = target - {'XAxis', 'YAxis'}
                if 'Maths' in target:
                    target = target - {'Saw', 'Ones', 'Cos', 'Sin', 'Zeros'}
                if 'Composed' in target:
                    target = target - {'Int32', 'Float64', 'Uint8', 'Uint64'}

                actual = set(filtered_mdf.channels_db) - {'t', 'time'}

                if 'Int16Array' in actual:
                    actual = actual - {'XAxis', 'YAxis'}
                if 'Maths' in actual:
                    actual = actual - {'Saw', 'Ones', 'Cos', 'Sin', 'Zeros'}
                if 'Composed' in actual:
                    actual = actual - {'Int32', 'Float64', 'Uint8', 'Uint64'}

                self.assertTrue(actual == target)

                equal = True

                with MDF(input_file, memory=memory) as mdf:

                    for name in channel_list:
                        original = mdf.get(name)
                        filtered = filtered_mdf.get(name)
                        if not np.array_equal(
                                original.samples,
                                filtered.samples):
                            equal = False
                        if not np.array_equal(
                                original.timestamps,
                                filtered.timestamps):
                            equal = False

                self.assertTrue(equal)

    def test_save(self):
        print("MDF save tests")

        compressions = [0, 1, 2]
        overwrite_enables = [True, False]
        for compression, memory, overwrite in product(compressions, MEMORY, overwrite_enables):

            for mdfname in os.listdir('tmpdir_demo'):
                input_file = os.path.join('tmpdir_demo', mdfname)
                if MDF(input_file).version == '2.00':
                # if MDF(input_file).version < '4.10':
                    continue
                print(input_file, compression, memory,overwrite)
                with MDF(input_file, memory=memory) as mdf:
                    out_file = mdf.save('tmp', compression=compression, overwrite=overwrite)
                    print(out_file)

                equal = True

                with MDF(input_file, memory=memory) as mdf, \
                        MDF(out_file, memory=memory) as mdf2:

                    for name in set(mdf.channels_db) - {'t', 'time'}:
                        original = mdf.get(name)
                        converted = mdf2.get(name)
                        if not np.array_equal(
                                original.samples,
                                converted.samples):
                            equal = False
                            print("NE", name, input_file, memory)
                            print(original)
                            print(converted)

                            os.rename(out_file, out_file+'err')
                            raise ValueError(1)
                        if not np.array_equal(
                                original.timestamps,
                                converted.timestamps):
                            equal = False

                self.assertTrue(equal)

    def test_save_array(self):
        print("MDF save array tests")

        compressions = [0, 1, 2]
        overwrite_enables = [True, False]
        for compression, memory, overwrite in product(compressions, MEMORY, overwrite_enables):

            for mdfname in os.listdir('tmpdir_array'):
                input_file = os.path.join('tmpdir_array', mdfname)
                print(input_file, compression, memory, overwrite)
                with MDF(input_file, memory=memory) as mdf:
                    out_file = mdf.save('tmp', compression=compression)

                    mdf.save('a_tmp_{}_{}.mf4'.format(compression, memory), compression=compression, overwrite=overwrite)

                equal = True

                with MDF(input_file, memory=memory) as mdf, \
                        MDF(out_file, memory=memory) as mdf2:

                    for name in set(mdf.channels_db) - {'t', 'time'}:
                        original = mdf.get(name)
                        converted = mdf2.get(name)
                        if not np.array_equal(
                                original.samples,
                                converted.samples):
                            equal = False
                        if not np.array_equal(
                                original.timestamps,
                                converted.timestamps):
                            equal = False

                self.assertTrue(equal)

    def test_select(self):
        print("MDF select tests")

        for mdfname in os.listdir('tmpdir_demo'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_demo', mdfname)

                if MDF(input_file).version == '2.00':
                    continue

                print(input_file, memory)

                channels_nr = np.random.randint(1, len(CHANNELS_DEMO) + 1)

                channel_list = random.sample(list(CHANNELS_DEMO), channels_nr)

                selected_signals = MDF(input_file, memory=memory).select(channel_list)

                self.assertTrue(len(selected_signals) == len(channel_list))

                self.assertTrue(all(ch.name == name for ch, name in zip(selected_signals, channel_list)))

                equal = True

                with MDF(input_file, memory=memory) as mdf:

                    for selected in selected_signals:
                        original = mdf.get(selected.name)
                        if not np.array_equal(
                                original.samples,
                                selected.samples):
                            equal = False
                            print(input_file, memory)
                            print(selected)
                            print(original)
                            raise ValueError(1)
                        if not np.array_equal(
                                original.timestamps,
                                selected.timestamps):
                            equal = False

                self.assertTrue(equal)

    def test_select_array(self):
        print("MDF select array tests")

        for mdfname in os.listdir('tmpdir_array'):
            for memory in MEMORY:
                input_file = os.path.join('tmpdir_array', mdfname)

                channels_nr = np.random.randint(1, len(CHANNELS_ARRAY) + 1)

                channel_list = random.sample(list(CHANNELS_ARRAY), channels_nr)

                selected_signals = MDF(input_file, memory=memory).select(channel_list)

                self.assertTrue(len(selected_signals) == len(channel_list))

                self.assertTrue(all(ch.name == name for ch, name in zip(selected_signals, channel_list)))

                equal = True

                with MDF(input_file, memory=memory) as mdf:

                    for selected in selected_signals:
                        original = mdf.get(selected.name)
                        if not np.array_equal(
                                original.samples,
                                selected.samples):
                            equal = False
                        if not np.array_equal(
                                original.timestamps,
                                selected.timestamps):
                            equal = False

                self.assertTrue(equal)


if __name__ == '__main__':
    unittest.main()
